import requests, json, os, csv, sys


def apiCall(cust_or_devs,offset,customer_id): 
    # here we use requests to call the central api for all customer details and returns all data gathered. 
    url_base_msp = 'https://eu-apigw.central.arubanetworks.com/msp_api'
    
    if cust_or_devs == 1:
        api_url = url_base_msp +'/v1/customers' + '?access_token=' + access_token + '&limit=50&offset=' + offset
    else:
        api_url = url_base_msp+'/v1/customers/' + customer_id + '/devices?access_token=' + access_token                                          
    
    api_response = requests.get(api_url)

    if api_response.status_code == 200:                  # if there's an issue getting the API data an error is thrown and the program exits
        return api_response.json()
    else:
        error_responses = {400: 'API Call - Error 400 - Bad Request', 401: 'API Call - Error 401 - Unauthorized - Access Token Expired?', 403: 'API Call - Error 403 - Forbidden', 
                            404: 'API Call - Error 404 - Not Found', 413: 'API Call - Error 413 - Request-size limit exceeded', 429: 'API Call - Error 429 - API rate limit exceeded',
                            500: 'API Call - Error 500 - Internal Server Error'}
        log(error_responses[api_response.status_code])
        exit()
    
def getCustomerIds(customers_json):
    # extract the customer IDs from the API call data and return as a list 

    customers_ids = []
    customers = customers_json['customers']
    total_customers = customers_json['total']

    for customer in customers:
        customers_ids.append(customer['customer_id'])
    
    customers_ids.pop()
    return customers_ids, int(total_customers)

def getCustomerDevices(customers_ids):
    # using the list of customer IDs we ruqery the API for the list of devices for each
    data = []

    for customer_id in customers_ids:
        devices_licenses = apiCall(2,0,customer_id)

        for devs_lics in devices_licenses['deviceList']['devices']:
            customer_name = devs_lics['customer_name']
            mac_address = devs_lics['macaddr']
            serial = devs_lics['serial']
            model = devs_lics['model']
            services = devs_lics['services']
            data.append([customer_id,customer_name,mac_address,serial,model,services])

    return (data)

def writeToFile(data):
    # open a file for writing
    report_name = 'devices_licenses.csv'
    report_file = open( os.getcwd() + '/' + report_name, 'w', newline='')

    # create the csv writer object
    csvwriter = csv.writer(report_file)
    header = ["Customer name","Customer ID","MAC address","Serial", "Model","Subscription","Services"]
    csvwriter.writerow(header)

    for devices in data:
        services = ""
        subscription = "Not susbscribed"
        a = 1 
        services_list = devices[5]
        for service in services_list:
            if service == "dm":
                services = ""
                subscription = "Subscribed"
            else:
                if a != len(services_list):
                    services += service + ", "
                else:
                    services += service
            a+=1
        csvwriter.writerow([devices[1],devices[0],devices[2],devices[3],devices[4],subscription,services])

    report_file.close()
    log ("Script finished OK")

def log(msg):
    print('msp_cust_devices_script # {}'.format(msg))

if __name__ == "__main__":
    access_token = input("Enter access token:")
    api_data = apiCall(1,'0',0)                                 # perform 1st API call with an offset of 0. This returns customer details starting at 1st customer.
    request = getCustomerIds(api_data)                          # strip the customer IDs from the data
    request_data = request[0]                                   # store customer IDs in request_data vairable so we can add more customer IDs to it
    
    while request[1] > len(request_data):                       # while the number of customer found is greater than the number of customer ID found, do mroe API calls to get more customer IDs
        api_data = apiCall(1,'50',0)                            # perform another API call with an offset of 50. This returns the next (upto) 50 customers       
        more_request_data = getCustomerIds(api_data)            # strip the customer IDs from the data and add to list request_data
        request_data.extend(more_request_data[0])

    data = getCustomerDevices(request_data)                     # using the customer IDs, get the list of devices for each customer
    log('Exporting {} customers devices to csv'.format(request[1]))
    writeToFile(data)   